# -*- coding: utf-8 -*-
# @package Index Sentinel Files
"""
Example:

        $ python -m doctest -v Index.py
"""

import os,sys
sys.path.append(os.path.join(os.path.expandvars("%userprofile%"),".snap\snap-python"))
import snappy,numpy
from snappy import HashMap,jpy,Product,ProductData,ProductIO,ProductUtils
from tools import (tools)
from readProduct import (readProduct)

class Index(object):
    """ Class for Sentinel satellites configuration
    Test Case
    >>> Files = tools('MSIL1C')
    >>> dir="make"
    >>> platform='Sentinel-2'
    >>> list_files=Files.findFilesSentinel(dir,platform,3)
    >>> print 'There is/are %d ' % len(list_files) + platform + ' file(s)'
    >>> Read=readProduct(dir,platform)
    >>> Ind="NBAI"
    >>> Indeex = Index("60", Ind)
    >>> for image in list_files:
    ...     file = dir + "/" + image
    ...     print"Applying Index " + Ind + "... ", image
    ...     product=Read.ReadFilesSentinel(file,2)
    ...     Indeex.Index(product, file)

     """

    def __init__(self,ResolSize,IndexType):
        """The constructor Initialize Sentinel Data.

        Args:
            self: The object pointer.
            ResolSize (str): destination directory.
            IndexType (str): # NDVI  - Normalized Difference Vegetation Index, NDWI - Normalized Difference Water Index, NBAI - Normalized Built-up Area Index

        Returns:
            pointer: The object pointer.
        """
        self.resolution=ResolSize
        self.Index_Type=IndexType
        self.typeNDVI = "NDVI"
        self.typeNDWI = "NDWI"
        self.typeNBAI = "NBAI"
        self.HashMap = jpy.get_type('java.util.HashMap')

    def Index(self,product, file):
        """Index Files Sentinel

         Args:
             self (pointer): The object pointer.
             product (bin): Metadata Sentinel.
             file (str): local path to image.

         """
        width = product.getSceneRasterWidth()
        height = product.getSceneRasterHeight()
        product_name = product.getName()

        if (self.Index_Type == self.typeNDVI):
            print"Entre NDVI..."
            type=self.typeNDVI
            red_band = product.getBand('B4')
            nir_band = product.getBand('B8')
        if (self.Index_Type == self.typeNDWI):
            print"Entre NDWI..."
            type=self.typeNDWI
            red_band = product.getBand('B11')
            nir_band = product.getBand('B8')
        if (self.Index_Type == self.typeNBAI):
            print"Entre NBAI..."
            type=self.typeNBAI
            do_band = product.getBand('B12')
            nir_band = product.getBand('B8')
            red_band = product.getBand('B2')
            do_row = numpy.zeros(width, dtype=numpy.float32)

        output_product = Product(self.Index_Type, self.Index_Type, width, height)
        ProductUtils.copyGeoCoding(product, output_product)
        output_band = output_product.addBand(type, ProductData.TYPE_FLOAT32)

        fileOut = file[:-4] + '_' + self.Index_Type + '.dim'
        output_product_writer = ProductIO.getProductWriter('BEAM-DIMAP')
        output_product.setProductWriter(output_product_writer)
        output_product.writeHeader(fileOut)

        red_row = numpy.zeros(width, dtype=numpy.float32)
        nir_row = numpy.zeros(width, dtype=numpy.float32)

        if (self.Index_Type == self.typeNDVI) or (self.Index_Type == self.typeNDWI):
            print"Entre NDVI o NDWI FOR..."
            for y in xrange(height):
                red_row = red_band.readPixels(0, y, width, 1, red_row)
                nir_row = nir_band.readPixels(0, y, width, 1, nir_row)
                ndvwi = (nir_row - red_row) / (nir_row + red_row)
                output_band.writePixels(0, y, width, 1, ndvwi)

        if (self.Index_Type == self.typeNBAI):
            print"Entre NBAI FOR..."
            for y in xrange(height):
                red_row = red_band.readPixels(0, y, width, 1, red_row)
                nir_row = nir_band.readPixels(0, y, width, 1, nir_row)
                do_row = do_band.readPixels(0, y, width, 1, do_row)
                nbai = (do_row - nir_row / red_row) / (do_row + nir_row / red_row)
                output_band.writePixels(0, y, width, 1, nbai)

        output_product.closeIO()