# -*- coding: utf-8 -*-
# @package JP2ToJPGSubSet Sentinel Files
"""
Example:

$ python -m doctest -v JP2ToJPGSubSet.py
"""

import os,sys,numpy as np
from tools import (tools)
from osgeo import gdal,ogr,osr


class JP2ToJPG_SubSet(object):
    """ Class for Sentinel satellites configuration
    Test Case
    >>> Files = tools("all") #IW_SLC,IW_GRDH,MSIL1C
    >>> dir="make/"
    >>> platform='Sentinel-2'
    >>> list_files=Files.findFilesSentinel(dir,platform,2)#4 for resample_subset.dim
    >>> print 'There is/are %d ' % len(list_files) + platform + ' file(s)'
    >>> lat1=20.795
    >>> lon1=-103.714
    >>> lat2=20.165
    >>> lon2=-103.086
    >>> TypeMerge="RGB"
    >>> Convert=JP2ToJPG_SubSet(TypeMerge)
    >>> for image in list_files:
    ... file = dir + "/"+ image
    ... products = Files.search_band_combination(file,TypeMerge)
    ... print"Applying Convert JP2 To JPEG and SubSet... ", image
    ... Convert.Convert_SubSet(lat1, lon1, lon2, lat2, products)
    """

    def __init__(self,TypeMerge):
        """The constructor Initialize Sentinel Data.

        Args:
        self: The object pointer.
        TypeMerge (str): RGB

        Returns:
        pointer: The object pointer.
        """
        self.ext=".jpg"
        self.SubSet="_SubSet"
        self.Type_Merge="_"+TypeMerge

    def Convert_SubSet(self, lat1, lon1, lon2, lat2, products):
        """Index Files Sentinel

         Args:
             self (pointer): The object pointer.
             lat1 (float): World Geodetic System 1984.
             lon1 (float): World Geodetic System 1984.
             lon2 (float): World Geodetic System 1984.
             lat2 (float): World Geodetic System 1984.
             files (str): local path to image.

         """
        for i in products:
            outDS=i[:-4]+self.SubSet+self.ext
            translate = 'gdal_translate -projwin %s %s %s %s -projwin_srs WGS84 -ot Byte -scale 0 4096 0 255 -of JPEG %s %s' % (lon1, lat1, lon2, lat2, i, outDS)
            os.system(translate)