# -*- coding: utf-8 -*-
# @package Merge Sentinel Files
"""
Example:

        $ python -m doctest -v Merge.py
"""

import os,sys,cv2
from PIL import Image
from os.path import expanduser
home = expanduser("~")
a=os.path.join(os.path.expandvars(home),".snap/snap-python/")
sys.path.append(a)
import snappy,numpy
from snappy import jpy,ProductData,ProgressMonitor,ProductUtils
from osgeo import ogr
from osgeo import gdal
from osgeo.gdalconst import *
from tools import (tools)
from readProduct import (readProduct)

class Merge(object):
    """ Class for Sentinel satellites configuration
    Test Case
    >>> Files = tools("all") #IW_SLC,IW_GRDH,MSIL1C
    >>> dir="make/"
    >>> platform='Sentinel-2'
    >>> list_files=Files.findFilesSentinel(dir,platform,4)#4 for resample_subset.dim
    >>> print 'There is/are %d ' % len(list_files) + platform + ' file(s)'
    >>> Read=readProduct(dir,platform)
    >>> TypeMerge="RGB"
    >>> Mergee=Merge("60",TypeMerge)
    >>> for image in list_files:
    ...     file = dir + image
    ...     print"Applying SubSet... ", image
    ...     product=Read.ReadFilesSentinel(file,2)#The flag 2 ReadFiles .dim
    ...     Mergee.merges(product, file)
     """

    def __init__(self,TypeMerge):
        """The constructor Initialize Sentinel Data.

        Args:
            self: The object pointer.
            TypeMerge (str): RGB

        Returns:
            pointer: The object pointer.
        """
        self.ext=".jpg"
        self.Type_Merge="_"+TypeMerge
        self.looks = jpy.get_type('org.esa.snap.core.datamodel.quicklooks.QuicklookGenerator')
        self.File = jpy.get_type('java.io.File')

    #def merges(self, product, file):
    def merges(self, product, products,file):
        """Index Files Sentinel

         Args:
             self (pointer): The object pointer.
             product (bin): Metadata Sentinel.
             file (str): local path to image.

         """
        ############################################################################
        #code ESA jpg
        blue = product.getBand('B2')
        green = product.getBand('B3')
        red = product.getBand('B4')

        RGB = [red, green, blue]

        info = ProductUtils.createImageInfo(RGB, True, ProgressMonitor.NULL)
        image = ProductUtils.createRgbImage(RGB, info, ProgressMonitor.NULL)

        salida = file[:-5] + self.Type_Merge + self.ext
        savefile = self.File(salida)
        self.looks.writeImage(image, savefile)
        ############################################################################
        ##OpenCV using .SAFE Images and searchBandsCombination
        dataset1=cv2.imread(products[2], cv2.IMREAD_ANYCOLOR)#B2 B cv2.IMREAD_ANYCOLOR
        print"B ",products[2]
        dataset2=cv2.imread(products[1], cv2.IMREAD_ANYCOLOR)#B4 R
        print"R ", products[1]
        dataset3=cv2.imread(products[0], cv2.IMREAD_ANYCOLOR)#B3 G
        print"G ", products[0]
        ##sys.exit()
        salida = file[:-5] + self.Type_Merge
        image = cv2.merge([dataset1, dataset3, dataset2])
        cv2.imwrite(salida + 'OpenCV.png', image)
        ############################################################################
        TotImage = len(products)
        # Get Raster
        dataset1 = gdal.Open(products[2], GA_ReadOnly)#B
        dataset2 = gdal.Open(products[1], GA_ReadOnly)#R
        dataset3 = gdal.Open(products[0], GA_ReadOnly)#G
        # Get metadata raster
        projectionfrom = dataset1.GetProjection()
        geotransform = dataset1.GetGeoTransform()
        band1 = dataset1.GetRasterBand(1)
        xsize = band1.XSize
        ysize = band1.YSize
        datatype1 = band1.DataType
        band2 = dataset2.GetRasterBand(1)
        datatype2 = band2.DataType
        band3 = dataset3.GetRasterBand(1)
        datatype3 = band3.DataType
        # Reading the raster values
        gtiff = gdal.GetDriverByName('GTiff')
        #output_file = file[:-5] + "_" + self.Type_Merge + ".tif"
        output_file = file[:-5] + self.Type_Merge + ".tif"
        values1 = band1.ReadRaster(0, 0, xsize, ysize, xsize, ysize, datatype2)
        values2 = band2.ReadRaster(0, 0, xsize, ysize, xsize, ysize, datatype3)
        values3 = band3.ReadRaster(0, 0, xsize, ysize, xsize, ysize, datatype1)
        # Combination bands
        output_dataset = gtiff.Create(output_file, xsize, ysize, TotImage, GDT_Int16)
        output_dataset.SetProjection(projectionfrom)
        output_dataset.SetGeoTransform(geotransform)
        output_dataset.GetRasterBand(1).WriteRaster(0, 0, xsize, ysize, values1)
        output_dataset.GetRasterBand(2).WriteRaster(0, 0, xsize, ysize, values2)
        output_dataset.GetRasterBand(3).WriteRaster(0, 0, xsize, ysize, values3)
        output_dataset = None


Files = tools("all") #IW_SLC,IW_GRDH,MSIL1C
dir="make/"
platform='Sentinel-2'
list_files=Files.findFilesSentinel(dir,platform,2)#4 for resample_subset.dim
print 'There is/are %d ' % len(list_files) + platform + ' file(s)'
Read=readProduct(dir,platform)
TypeMerge="RGB"
Mergee=Merge(TypeMerge)
for image in list_files:
    file = dir + image
    products = Files.search_band_combination(file, TypeMerge)
    print"Applying Merge "+TypeMerge+"... ", image
    product=Read.ReadFilesSentinel(file,2)#The flag 2 ReadFiles .dim
    #Mergee.merges(product, file)#for resample_subset.dim
    Mergee.merges(product, products,file)#for .SAFE
