# -*- coding: utf-8 -*-
# @package SubSet Sentinel Files
"""
Example:

        $ python -m doctest -v SubSet.py
"""



import os,sys
from os.path import expanduser
home = expanduser("~")
a=os.path.join(os.path.expandvars(home),".snap/snap-python/")
sys.path.append(a)
import snappy
from snappy import HashMap,jpy,Product,ProductData,ProductIO,ProductUtils,GPF
from tools import (tools)
from readProduct import (readProduct)

class SubSet(object):
    """ Class for Sentinel satellites configuration
    Test Case
    >>> Files = tools('MSIL1C')
    >>> dir="make/"
    >>> platform='Sentinel-2'
    >>> list_files=Files.findFilesSentinel(dir,platform,3)
    >>> print 'There is/are %d ' % len(list_files) + platform + ' file(s)'
    >>> Read=readProduct(dir,platform)
    >>> widthX="0"
    >>> heigthX="0"
    >>> widthY="588"
    >>> heigthY="636"
    >>> SubSeet = SubSet("60", widthX, heigthX, widthY, heigthY)
    >>> for image in list_files:
    ...     file = dir + "/" + image
    ...     print"Applying SubSet... ", image
    ...     product=Read.ReadFilesSentinel(file,2)
    ...     SubSeet.subSet(product, file)

     """
    def __init__(self,ResolSize,widthX,heigthX,widthY,heigthY):
        """The constructor Initialize Sentinel Data.

        Args:
            self: The object pointer.
            ResolSize (str): destination directory.
            widthX (str): Scene start X
            heigthX (str):Scene start Y
            widthY (str): Scene end X
            heigthY (str):Scene end Y

        Returns:
            pointer: The object pointer.
        """
        self.resolution=ResolSize
        self.Width_X=widthX
        self.Heigth_X=heigthX
        self.Width_Y=widthY
        self.Heigth_Y=heigthY
        self.Type_file="_SubSet"
        self.HashMap = jpy.get_type('java.util.HashMap')

    def subSet(self,product, file):
        """subSet Files Sentinel

         Args:
             self (pointer): The object pointer.
             product (bin): Metadata Sentinel.
             file (str): local path to image.

         """
        params = self.HashMap()
        params.put('copyMetadata', True)
        params.put('region', "%s,%s,%s,%s" % (self.Width_X, self.Heigth_X, self.Width_Y, self.Heigth_Y))
        subset = GPF.createProduct('Subset', params, product)
        salida = file[:-4] + self.Type_file + ".dim"
        ProductIO.writeProduct(subset, salida, "BEAM-DIMAP")

