# -*- coding: utf-8 -*-
# @package Tools Sentinel
"""
Example:

        $ python -m doctest -v tools.py
"""
import os,sys


class tools(object):
    """ Class for Sentinel satellites configuration
    Test Case
    >>> Files = tools('all')
    >>> dir='make'
    >>> platform='Sentinel-2'
    >>> list_files=Files.findFilesSentinel(dir,platform)
    >>> print 'There is/are %d ' % len(list_files) + platform + ' file(s)'

     """
    def __init__(self,type):
        """The constructor Initialize Sentinel Data.

        Args:
            self: The object pointer.
            type (str): sentinel product type {all, SM_SLC, SM_GRDF, SM_GRDH, SM_GRDM, IW_SLC, IW_GRDH, IW_GDRM, IW_RAW, EW_SLC, EW_GRDH, EW_GRDM, WV_SLC, WV_GRDM, MSIL1C, OPER_PRD }.

        Returns:
            pointer: The object pointer.
        """
        self.product_type=type
        self.namfil1 = 'S1A_'
        self.namfil2 = 'S1B_'
        self.namfil3 = 'S2A_'
        self.namfil4 = 'S2B_'


    def findFilesSentinel(self,dir,platform,flag):
        """Find Files Sentinel

        Args:
            self (pointer): The object pointer.
            dir (str): destination directory.
            platform (str): Type platform Sentinel-1 or Sentinel-2

        Returns:
            OrderedDict: Sentinel Product list found.

        """
        if(flag==1):
            ext = ".zip"
        if(flag==2):
            ext = ".SAFE"
        if(flag==3):
            ext="_Resample_60.dim"
        if(flag==4):
            ext="_Resample_60_SubSet.dim"
        files = os.listdir(dir)
        if (self.product_type == "all"):
            if platform == "Sentinel-1":
                nam1=self.namfil1
                nam2=self.namfil2
                #files = [f for f in files if f.startswith(nam1) and f.endswith(ext) or f.startswith(nam2)]
            if platform == "Sentinel-2":
                nam1=self.namfil3
                nam2=self.namfil4
            files = [f for f in files if f.startswith(nam1) and f.endswith(ext) or f.startswith(nam2)]
            if (len(files) == 0):
                print"Files not found..."
                sys.exit()
        else:
            if platform=="Sentinel-1":
                nam1 = self.namfil1 + self.product_type
                nam2 = self.namfil2 + self.product_type
                #files = [f for f in files if f.startswith(nam1) and f.endswith(ext) or f.startswith(nam2)]
            if platform=="Sentinel-2":
                nam1 = self.namfil3 + self.product_type
                nam2 = self.namfil4 + self.product_type
            files = [f for f in files if f.startswith(nam1) and f.endswith(ext) or f.startswith(nam2)]
            if (len(files) == 0):
                print"Files not found..."
                sys.exit()
        return files

    def search_band_combination(self, sentinel_image,Type_Merge):
        Granule = "/GRANULE"
        path = sentinel_image + Granule
        file = os.listdir(path)
        path = path + "/" + file[0]
        Img_Data = "/IMG_DATA"
        path = path + Img_Data
        file = os.listdir(path)
        products = []
        if Type_Merge == "RGB":
            ext1 = "B04.jp2"
            ext2 = "B03.jp2"
            ext3 = "B02.jp2"
            raster = [f for f in file if f.endswith(ext1) or f.endswith(ext2) or f.endswith(ext3)]
        for raster in raster:
            product = path + "/" + raster
            a = os.path.join(os.getcwd(), product)
            #print"RUTA.....",a
            #sys.exit()
            products.append(a)
        # combinations_bands_sentinel(sentinel_image, Type_Merge, products)
        return products
