#!/usr/bin/env python
# -*- coding: utf-8 -*-

#import cv2
import sys
import numpy as np
import gdal
from osgeo import osr

def geoint_pr_normalizedIndex(b1_fn, b2_fn, outfile):
	'''!
		Normalized Difference Index (ND*I)
		@param b1_fn String: band #1 (S2_B*) Image file name
		@param b2_fn String: Band #2 (S2_B*) Image file name
		@param outfile String: processed image file name
	'''
	b1 = gdal.Open(b1_fn)
	b2 = gdal.Open(b2_fn)

	rows, cols, geotransform = b1.RasterYSize, b1.RasterXSize, b1.GetGeoTransform()

	# Read the input bands as numpy arrays.
	np_b1 = b1.GetRasterBand(1).ReadAsArray(0,0,b1.RasterXSize, b1.RasterYSize)
	np_b2 = b2.GetRasterBand(1).ReadAsArray(0,0,b2.RasterXSize, b2.RasterYSize) 

	# Convert the np arrays to 32-bit floating point to make sure division will occur properly.
	np_b1_as32 = np_b1.astype(np.float32)
	np_b2_as32 = np_b2.astype(np.float32)

	# Calculate the NDVI formula.
	np.seterr(divide='ignore', invalid='ignore')
	numerator = np.subtract(np_b1_as32, np_b2_as32)
	denominator = np.add(np_b1_as32, np_b2_as32)
	result = np.divide(numerator, denominator)
	result = np.multiply((result + 1), (2**7 - 1))

	driverTiff = gdal.GetDriverByName('GTiff')
	output = driverTiff.Create(outfile, cols, rows, 1, gdal.GDT_Byte)
	output.GetRasterBand(1).SetNoDataValue(-99)
	output.GetRasterBand(1).WriteArray(result)
	output.GetRasterBand(1).FlushCache()
	output.SetGeoTransform(geotransform)
	wkt = b1.GetProjection()
	sr = osr.SpatialReference()
	sr.ImportFromWkt(wkt)

	output.SetProjection(sr.ExportToWkt())
	output = None


def main(argv):
	geoint_pr_normalizedIndex(argv[1], argv[2], argv[3])

if __name__ == "__main__":
	main(sys.argv)
